`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Engineer: Scott Gravenhorst
// email: music.maker@gte.net
// 
// Create Date:     09-03-2011
// Design Name:     Digital Waveguide Polyphonic Synthesizer
// Module Name:     Xarp
// Project Name:    Digital Waveguide Polyphonic Synthesizer
// Target Devices:  Spartan-3E
//
// Multiple Karplus-Strong String Model, 61 separate strings, each which can 
// vibrate simultaneously with the others.  Xarp consists of 2 harps which operate
// in parallel with each other.
//
// Sample rate is nominally 96.0 kHz
//
//    ********************************************************************************
//    * *** NOTE *** When using this instrument, the sustain pedal must be depressed *
//    * for sustain.  If all you get is "thup thup", that's likely the problem.      *
//    ********************************************************************************
//
//////////////////////////////////////////////////////////////////////////////////
//
// For Spartan-3E (XC3S500E-4FG320)
//
//////////////////////////////////////////////////////////////////////////////////
//
//                                PUSH BUTTON CONTROLS:
//
//////////////////////////////////////////////////////////////////////////////////
//
// ver_a: - first cut
//
// ver_b: - changed tuning ROMs to move lowest key from E to C (8 notes higher)
//
// ver_c: - add lagrange interpolators for fractional delay
//
// ver_d: - Add support for patch editor.  At least for reflection filter a0 values
//          and MIDI channel.
//        * Fix tuning error bug, wrap detector set to go off one count early
//          making esp. higher strings sound sharp.
//        * Fix tuning ROMs, they were starting at C# instead of C
//        * Changed HARP_RAM to 13 K instead of 18K (frees up multipliers)
//        * Changed HARP_RAM to true dual port.
//
// ver_e: - Raise sample rate from 48 kHz to 96 kHz to raise the output pitch by
//          one octave.  This requires that 2 harps operate in parallel to halve
//          the number of state machine clocks.  HARP_RAM will be used in true
//          dual port mode to accomplish the double harp.  The harps will be organized
//          as a bass harp and a treble harp where the first 32 strings will be in the 
//          bass harp and the upper 32 (top 3 of which are not used) will be in the 
//          treble harp.  Double IIR filter, double lagrange interpolators and modified
//          pluck logic are required.  Actual block RAM usage turned out to be less than 
//          13K so 5K was removed freeing up the multiplier needed for doubled filters
//          etc.
//
//////////////////////////////////////////////////////////////////////////////////
module Xarp ( 

            CLK_50MHZ, 

            led,
            BTN_N,BTN_W,BTN_S, 
            spi_sck, spi_sdi, spi_dac_cs, spi_sdi, spi_sdo, spi_rom_cs, 
            spi_amp_cs, spi_adc_conv, spi_dac_cs, spi_amp_shdn, spi_dac_clr,
            strataflash_oe, strataflash_ce, strataflash_we,
            platformflash_oe,
            Raw_MIDI_In,
            TTY_In 

            );
            
  input  CLK_50MHZ;                       // 50 MHz soldered clock.
  output [7:0] led;                       // 8 LEDs on the board.
  input  Raw_MIDI_In;                     // MIDI input
  input  TTY_In;                          // TTY input

  input BTN_N;
  input BTN_W;
  input BTN_S;

  output spi_sck;
  output spi_sdi;
  input spi_sdo;
  output spi_rom_cs;
  output spi_amp_cs;
  output spi_adc_conv;
  output spi_dac_cs;
  output spi_amp_shdn;
  output spi_dac_clr;
  output strataflash_oe;
  output strataflash_ce;
  output strataflash_we;
  output platformflash_oe;

/////////////////////////////////////////////////////////////////////////////////////

  wire CLK_50MHZ;
  wire [7:0] led;
  wire TTY_In;
  wire Raw_MIDI_In;
  wire clk;              // 48 MHZ system clock 
  wire BTN_W;

  wire BTN_N;

  wire BTN_S;


/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////
// 48.0 MHz clock generator
  
  DCM_48 DCMCLK0 ( .CLKIN_IN( CLK_50MHZ ), .CLKFX_OUT( clk ) );

  
  wire [3:0] rstd;                 // POR delay
  wire       reset;                // POR/User reset

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

// mcu
  wire        interrupt;
  wire        interrupt_ack;
  wire [9:0]  address;             // wires to connect address lines from uC to ROM
  wire [17:0] instruction;         // uC data lines, need connection between uC and ROM
  wire [7:0]  out_port;            //
  wire [7:0]  in_port;             // 
  wire [7:0]  port_id;
  wire        read_strobe;
  wire        write_strobe;
  reg  [7:0]  in_port_reg;         // hold data for mcu
  assign in_port = in_port_reg;

// MIDI & TTY receivers
  wire [7:0] rx0data;
  wire [7:0] rx1data;
  wire       rx0ready_status;
  wire       rx1ready_status;
  wire       reset_rx0ready_status;
  wire       reset_rx1ready_status;
  
  wire       resetsignal;

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// Synth signals

// Pluck logic:
  reg        Set_Pluck = 0;    //
  reg        Clr_Pluck = 0;    //
  reg  [5:0] Plucked_Str;      // String within the harp which was plucked
  wire       Pluck;            // Q from Pluck FF

// Output values from processing of MIDI data.
  reg [6:0] MOD_WHL = 7'd127;                      // a usable initial value
//  reg [6:0] JOY_X = 7'd62;                      // a usable initial value
//  reg [6:0] JOY_Y = 7'd120;                      // a usable initial value
  
  reg       SUSTAIN = 1'b0;

  reg [6:0] VEL;

  reg [4:0] STR  = 0;                            // used as string select
  reg [4:0] STR0 = 0;                            // used as string select
  reg [4:0] STR1 = 0;                            // used as string select
  reg [4:0] STR2 = 0;                            // used as string select
  reg [4:0] STR3 = 0;                            // used as string select
  
  reg [3:0] MIDI_CHAN = 0;

//////////////////////////////////////////////////////////////////////////////////////////////////
// POR delay FF chain - taken from Eric Brombaugh's code for the SPI DAC.
  FDCE rst_bit0 (.Q(rstd[0]), .C(clk), .CE(1'b1), .CLR(1'b0), .D(1'b1));
  FDCE rst_bit1 (.Q(rstd[1]), .C(clk), .CE(1'b1), .CLR(1'b0), .D(rstd[0]));
  FDCE rst_bit2 (.Q(rstd[2]), .C(clk), .CE(1'b1), .CLR(1'b0), .D(rstd[1]));
  FDCE rst_bit3 (.Q(rstd[3]), .C(clk), .CE(1'b1), .CLR(1'b0), .D(rstd[2]));
  assign reset = ~rstd[3] | BTN_N;

  

  // Tie off the flash enables to allow SPI to work
  assign strataflash_oe = 1'b1;
  assign strataflash_ce = 1'b1;
  assign strataflash_we = 1'b1;
  assign platformflash_oe = 1'b0;
  
// Tie off other SPI enables to isolate DAC
  assign spi_rom_cs = 1'b1;
  assign spi_amp_cs = 1'b1;
  assign spi_adc_conv = 1'b0;
  assign spi_amp_shdn = 1'b1;
  assign spi_dac_clr = 1'b1;

//////////////////////////////////////////////////////////////////////////////////////////////////
// instantiate the uC (kcpsm3) with it's ROM

  kcpsm3 MCU0 ( .address(address), .instruction(instruction), .port_id(port_id), 
    .write_strobe(write_strobe), .out_port(out_port), .read_strobe(read_strobe), .in_port(in_port), 
    .interrupt(interrupt), .interrupt_ack(interrupt_ack), 
    .reset(reset), .clk(clk) );  

  midictrl PSM_ROM0 ( .address(address), .instruction(instruction), .clk(clk) );
  
  // MIDI receiver.  31,250 Baud
  assign resetsignal = write_strobe && ( port_id == 8'hFF );  // When port_id == FF with write strobe, reset the UARTs

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////
                                                                                        ///
   wire interrupt0;                                                                     ///
   wire interrupt1;                                                                     ///
                                                                                        ///
// MIDI UART                                                                            ///
// UART code by Jim Patchell                                                            ///
  MIDIuartrx RX0 ( .dout(rx0data), .clk(clk), .reset(resetsignal), .rxd(Raw_MIDI_In),   ///
    .frame(), .overrun(), .ready(), .busy(), .CS(),                                     ///
    .interrupt(interrupt0), .interrupt_ack(interrupt_ack),                              ///
    .rxready_status(rx0ready_status),                                                   ///
    .reset_rxready_status(reset_rx0ready_status)                                        ///
    );                                                                                  ///
  /////// VERY IMPORTANT HARDWARE /////////////////////////////////////////////////////////
  // decode read port 01, send pulse to reset rxready flop                              ///
  // This allows the mcu to clear the rxready bit automatically just by reading rxdata. ///
  assign reset_rx0ready_status = (read_strobe == 1'b1) & (port_id[4:0] == 5'h01);       ///
                                                                                        ///
///////////////////////////////////////////////////////////////////////////////////////////
// TTY UART, 115.2 or 19.2 kilobuad (baudrate configured in module)                     ///
// UART code by Jim Patchell                                                            ///
  TTYuartrx RX1 ( .dout(rx1data), .clk(clk), .reset(resetsignal), .rxd(TTY_In),         ///
    .frame(), .overrun(), .ready(), .busy(), .CS(),                                     ///
    .interrupt(interrupt1), .interrupt_ack(interrupt_ack),                              ///
    .rxready_status(rx1ready_status),                                                   ///
    .reset_rxready_status(reset_rx1ready_status)                                        ///
    );                                                                                  ///
  /////// VERY IMPORTANT HARDWARE /////////////////////////////////////////////////////////
  // decode read port 04, send pulse to reset rxready flop                              ///
  // This allows the mcu to clear the rxready bit automatically just by reading rxdata. ///
  assign reset_rx1ready_status = (read_strobe == 1'b1) & (port_id[4:0] == 5'h09);       ///
                                                                                        ///
// common interrupt signal for both serial ports.                                       ///
// ISR gets to figure out which UART did it.                                            ///
  assign interrupt = (interrupt0 | interrupt1);                                         ///
                                                                                        ///
///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// DAC module by Eric Brombaugh

  parameter CYCLES = 12'd500;     // 96 kHz (48 MHz system clock)
//  parameter CYCLES = 12'd503;     // 96 kHz (48 MHz system clock)
//  parameter CYCLES = 12'd501;     // 96 kHz (48 MHz system clock)
//  parameter CYCLES = 12'd497;     // 96 kHz (48 MHz system clock)

  reg [11:0] DACreg = 12'h800;
  wire       DACena;

  spi_dac_out DAC ( .clk( clk ), .reset( reset ), 
                    .spi_sck( spi_sck ), .spi_sdo( spi_sdi ), .spi_dac_cs( spi_dac_cs ),
                    .ena_out( DACena ), .data_in( DACreg ), .cycles( CYCLES ) );

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// Tuning ROMS

  wire signed [17:0] LenFracROM_bass;
  fraction_bass_ROM TUN_FRAC0 ( .a( STR3 ), .spo( LenFracROM_bass ) );   // pan value for the lagrange interpolator

  wire signed [17:0] LenFracROM_treb;
  fraction_treb_ROM TUN_FRAC1 ( .a( STR3 ), .spo( LenFracROM_treb ) );   // pan value for the lagrange interpolator


  wire [13:0] StrEndROM_bass;
  end_bass_ROM TUN_END0 ( .a( STR1 ), .spo( StrEndROM_bass ) );       // end address for each string

  wire [13:0] StrEndROM_treb;
  end_treb_ROM TUN_END1 ( .a( STR1 ), .spo( StrEndROM_treb ) );       // end address for each string


  wire [13:0] StrStartROM_bass;
  start_bass_ROM TUNE_START0 ( .a( STR2 ), .spo( StrStartROM_bass ) );// starting address for each string
    
  wire [13:0] StrStartROM_treb;
  start_treb_ROM TUNE_START1 ( .a( STR2 ), .spo( StrStartROM_treb ) );// starting address for each string
    
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//
//==//==//==//==//==//==//==//==//==//==//  WAVEGUIDE RAM   //==//==//==//==//==//==//==//==//==//==//==//==//
//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//==//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// String RAMs (delay lines)

  reg [13:0]  WGadr_bass = 0;                        // state machine waveguide address register
  reg [13:0]  WGadr_treb = 0;                        // state machine waveguide address register

  reg [13:0]  NEXT_WGadr_bass = 0;
  reg [13:0]  NEXT_WGadr_treb = 0;

  reg [13:0]  WGadr_bass_RAM [31:0];
  wire [13:0] WGadr_bass_RAM_out;
  assign WGadr_bass_RAM_out = WGadr_bass_RAM[STR3];
  
  reg [13:0] WGadr_treb_RAM [31:0];
  wire [13:0] WGadr_treb_RAM_out;
  assign WGadr_treb_RAM_out = WGadr_treb_RAM[STR3];
  
//////// Waveguide RAM ///////////////////////////////////////////////////////////////////////

  reg  HARP_RAM_WRT = 1'b0;

  wire signed [17:0] HARP_RAM_bass;
  reg  signed [17:0] HARP_RAM_bass_in;

  wire signed [17:0] HARP_RAM_treb;
  reg  signed [17:0] HARP_RAM_treb_in;

// true dual port block RAM
  HARP_RAM HARP_RAM0 (
    .clka( clk ), 
    .wea( HARP_RAM_WRT ), 
    .addra( WGadr_bass ), 
    .dina( HARP_RAM_bass_in ), 
    .douta( HARP_RAM_bass ), 
    .clkb( clk ), 
    .web( HARP_RAM_WRT ), 
    .addrb( WGadr_treb ), 
    .dinb( HARP_RAM_treb_in ), 
    .doutb( HARP_RAM_treb )
    );

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// Reflection point filters -  two filters are shared between all 60 strings.

// BASS
  wire signed [17:0] a0_bass;
  reg  signed [17:0] b1_bass = 0;

  reg [6:0] a0_bass_lo_RAM [31:0];
  reg [6:0] a0_bass_mi_RAM [31:0];
  reg [3:0] a0_bass_hi_RAM [31:0];

  assign a0_bass = {a0_bass_hi_RAM[STR0],a0_bass_mi_RAM[STR0],a0_bass_lo_RAM[STR0]};

  reg signed  [17:0] IIRz_bass [31:0];
  
  wire signed [17:0] IIR_bass_out;
  assign IIR_bass_out = IIRz_bass[STR1];

// TREBLE
  wire signed [17:0] a0_treb;
  reg  signed [17:0] b1_treb = 0;

  reg [6:0] a0_treb_lo_RAM [31:0];
  reg [6:0] a0_treb_mi_RAM [31:0];
  reg [3:0] a0_treb_hi_RAM [31:0];

  assign a0_treb = {a0_treb_hi_RAM[STR0],a0_treb_mi_RAM[STR0],a0_treb_lo_RAM[STR0]};

  reg signed  [17:0] IIRz_treb [31:0];
  
  wire signed [17:0] IIR_treb_out;
  assign IIR_treb_out = IIRz_treb[STR1];
   
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// LaGrange interpolator logic RAM and registers

  wire signed [17:0] FIRz_bass_RAM_out;
  reg  signed [17:0] FIRz_bass_RAM [31:0];
  assign FIRz_bass_RAM_out = FIRz_bass_RAM[STR3];

  reg  signed [17:0] p0_bass = 0;
  reg  signed [17:0] p1_bass = 0;  
  
  
  wire signed [17:0] FIRz_treb_RAM_out;
  reg  signed [17:0] FIRz_treb_RAM [31:0];
  assign FIRz_treb_RAM_out = FIRz_treb_RAM[STR3];

  reg  signed [17:0] p0_treb = 0;
  reg  signed [17:0] p1_treb = 0;
  
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// for EXval:

  reg signed [17:0] EXval_bass [31:0];         // waveguide EXval (excitation value) signal;
  wire signed [17:0] EXval_bass_RAM;
  assign EXval_bass_RAM = EXval_bass[STR2];

  reg signed [17:0] EXval_treb [31:0];         // waveguide EXval (excitation value) signal;
  wire signed [17:0] EXval_treb_RAM;
  assign EXval_treb_RAM = EXval_treb[STR2];

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// EX excite flags:
  
  reg [0:0] EX_bass_RAM [31:0];
  wire EX_bass;
  assign EX_bass = EX_bass_RAM[STR2];

  reg [0:0] EX_treb_RAM [31:0];
  wire EX_treb;
  assign EX_treb = EX_treb_RAM[STR2];

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// EXcnt:

  reg [12:0] EXcnt_bass [31:0];
  wire [12:0] EXcnt_bass_RAM;
  assign EXcnt_bass_RAM = EXcnt_bass[STR3];


  reg [12:0] EXcnt_treb [31:0];
  wire [12:0] EXcnt_treb_RAM;
  assign EXcnt_treb_RAM = EXcnt_treb[STR3];


  reg signed [17:0] EXvaltmp_bass = 0;       // temp reg for computing EXval[] per string
  reg signed [17:0] EXvaltmp_treb = 0;       // temp reg for computing EXval[] per string

  reg        [12:0] EXcnttmp_bass = 0;       // temp reg for computing EXcnt[] per string
  reg        [12:0] EXcnttmp_treb = 0;       // temp reg for computing EXcnt[] per string

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// Waveguide state machine

// 64 sign bits, one for each string
  reg        [0:0]  sign0_bass [31:0];      // used to invert sign for each excite to prevent "charging" the system.
  reg        [0:0]  sign0_treb [31:0];      // used to invert sign for each excite to prevent "charging" the system.

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// mini harp output registers:  (these are accumulator registers for each mini harp)
  reg signed [23:0] HARPsum = 0;

//////////////////////////////////////////////////////////////////////////////////////////////////


// Waveguide inputs
  reg signed [17:0] WGin_bass = 0;
  reg signed [17:0] WGinP_bass = 0;
  reg signed [17:0] WGinN_bass = 0;

  reg signed [17:0] WGin_treb = 0;
  reg signed [17:0] WGinP_treb = 0;
  reg signed [17:0] WGinN_treb = 0;


// Pulse width for pluck
  reg        [13:0] PULSlen_bass = 0;  
  reg        [13:0] PULSlen_treb = 0;  

// overflow detection:
//  wire oflo0;
//  assign oflo0 = (( EXvaltmp[17] ^ IIRz[17] ) & ( WGin[17] ^ EXvaltmp[17] ));

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// shared multipliers

  reg  signed [17:0] mA ;
  reg  signed [17:0] mB ;
  wire signed [35:0] mP ;
  wire signed [17:0] mPROD ;

  reg  signed [17:0] mPROD_reg ;
  assign mP = mA * mB ;
  assign mPROD = mP >>> 17 ;

  reg  signed [17:0] nA ;
  reg  signed [17:0] nB ;
  wire signed [35:0] nP ;
  wire signed [17:0] nPROD ;

  reg  signed [17:0] nPROD_reg ;
  assign nP = nA * nB ;
  assign nPROD = nP >>> 17 ;

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// STATE MACHINE REGISTERS

  reg [13:0] StrEnd_bass = 0;
  reg [13:0] StrStart_bass = 0;

  reg [13:0] StrEnd_treb = 0;
  reg [13:0] StrStart_treb = 0;

  reg [13:0] STRlen_bass;

  reg [13:0] STRlen_treb;

  reg  signed [17:0] FULL_WG_OUT_bass = 0;
  reg  signed [17:0] FULL_WG_OUT_treb = 0;

  reg signed [11:0] NEXT_DAC = 0;

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// STATE MACHINE

  reg no_leds = 0;                       // flip flop for LEDS on / LEDs off
  reg old_BTN_W = 0;
  always @ ( posedge clk )
    begin
    old_BTN_W <= BTN_W;
    if ( old_BTN_W != BTN_W ) no_leds = ~no_leds;
    end

////////////////////////////////////////////////////////////////////////////////////////////////////////
// state machine structure:

  reg [4:0] state = 0;
  reg run = 0;
  
  always @ ( posedge clk )
    begin
    if ( DACena )
      begin
      state     <= 0;                      // starting state
      run       <= 1;                      // tell state machine to run
      DACreg    <= {~NEXT_DAC[11],NEXT_DAC[10:0]};
      HARPsum   <= 0;                      // clear sum register
      STR       <= 4'h0;                   // Select string zero to start
      end
    
    else        // else if NOT DACena:
      
      begin
      if ( run )
        begin
        case ( state ) 
        
        //////////// start of string loop /////////////


        5'h00:
          begin  state <= state + 1;
          STR0 <= STR;
          STR1 <= STR;
          STR2 <= STR;
          STR3 <= STR;
          end
        
        5'h01:
          begin  state <= state + 1;
          WGadr_bass <= WGadr_bass_RAM_out;
          WGadr_treb <= WGadr_treb_RAM_out;

          StrStart_bass <= StrStartROM_bass;    // The starting address for this string.  StrStartROM is one of the tuning ROMs  
          StrStart_treb <= StrStartROM_treb;    // The starting address for this string.  StrStartROM is one of the tuning ROMs  

          StrEnd_bass   <= StrEndROM_bass;      // the ending address for this string.  StrEndROM is on of the tuning ROMs
          StrEnd_treb   <= StrEndROM_treb;      // the ending address for this string.  StrEndROM is on of the tuning ROMs

          b1_bass       <= 19'sh20000 - $signed({1'b0,a0_bass}); // b1 value (leaky integrator).  a0 is wires from the a0 ROM
          b1_treb       <= 19'sh20000 - $signed({1'b0,a0_treb}); // b1 value (leaky integrator).  a0 is wires from the a0 ROM

          p0_bass       <= 19'sh20000 - {1'b0,LenFracROM_bass};
          p0_treb       <= 19'sh20000 - {1'b0,LenFracROM_treb};

          p1_bass       <= LenFracROM_bass;
          p1_treb       <= LenFracROM_treb;
          end
        
        5'h02:                        // 
          begin  state <= state + 1;
          mA <= p1_bass;                   // the previous sample parameter
          mB <= FIRz_bass_RAM_out;         // the previous sample

          nA <= p1_treb;                   // the previous sample parameter
          nB <= FIRz_treb_RAM_out;         // the previous sample

          STRlen_bass <= StrEnd_bass - StrStart_bass;
          STRlen_treb <= StrEnd_treb - StrStart_treb;
          end

        5'h03:                        // 
          begin  state <= state + 1;
          mPROD_reg <= mPROD;
          nPROD_reg <= nPROD;
          
          mA <= p0_bass;                   // the current sample parameter
          mB <= HARP_RAM_bass;             // HARP_RAM is IIRin.  This signal is the waveguide output 

          nA <= p0_treb;                   // the current sample parameter
          nB <= HARP_RAM_treb;             // HARP_RAM is IIRin.  This signal is the waveguide output 

          FIRz_bass_RAM[STR3] <= HARP_RAM_bass;
          FIRz_treb_RAM[STR3] <= HARP_RAM_treb;
          end

        5'h04:                        // 
          begin  state <= state + 1;
          FULL_WG_OUT_bass <= mPROD + mPROD_reg;
          FULL_WG_OUT_treb <= nPROD + nPROD_reg;
          
          mA <= IIR_bass_out;               // the z value, also output, IIRz is wires to the IIRz dist. RAM
          mB <= b1_bass;                    // b1 value calculated from a0 value

          nA <= IIR_treb_out;               // the z value, also output, IIRz is wires to the IIRz dist. RAM
          nB <= b1_treb;                    // b1 value calculated from a0 value
          end

        5'h05:                        
          begin  state <= state + 1;
          mPROD_reg <= mPROD;
          nPROD_reg <= nPROD;

          mA <= FULL_WG_OUT_bass;             // HARP_RAM is IIRin.  This signal is the waveguide output 
          mB <= a0_bass;                      // a0 is wires from the a0 ROM

          nA <= FULL_WG_OUT_treb;             // HARP_RAM is IIRin.  This signal is the waveguide output 
          nB <= a0_treb;                      // a0 is wires from the a0 ROM
          end

        5'h06:
          begin  state <= state + 1;
          IIRz_bass[STR1] <= mPROD_reg + mPROD;  // save the output of the filter in RAM
          IIRz_treb[STR1] <= nPROD_reg + nPROD;  // save the output of the filter in RAM

          EXvaltmp_bass <= EXval_bass_RAM;    // sets default value for EXvaltmp in case next state's logic doesn't change it.
          EXvaltmp_treb <= EXval_treb_RAM;    // sets default value for EXvaltmp in case next state's logic doesn't change it.

          EXcnttmp_bass <= EXcnt_bass_RAM;    // loads the stored counter state for each string in each miniharp
          EXcnttmp_treb <= EXcnt_treb_RAM;    // loads the stored counter state for each string in each miniharp
          end

        5'h07:     // multiply lowpass filt out by mod wheel to provide input for the waveguide.
          begin  state <= state + 1;
          mA <= $signed({7'b0111111,MOD_WHL,4'b1111});   // linear atten. from mod wheel
          mB <= IIR_bass_out;

          nA <= $signed({7'b0111111,MOD_WHL,4'b1111});   // linear atten. from mod wheel
          nB <= IIR_treb_out;
          end

        5'h08:     // mPROD is the attenuated lowpass filter output. 
          begin  state <= state + 1;

          PULSlen_bass <= (STRlen_bass >> 3);          // 1/8 wavelength (approx)
          PULSlen_treb <= (STRlen_treb >> 3);          // 1/8 wavelength (approx)

          WGinP_bass <= -mPROD + EXvaltmp_bass; // this will generate a positive pulse
          WGinN_bass <= -mPROD - EXvaltmp_bass; // this will genarate a negative pulse

          WGinP_treb <= -nPROD + EXvaltmp_treb; // this will generate a positive pulse
          WGinN_treb <= -nPROD - EXvaltmp_treb; // this will genarate a negative pulse

          HARPsum <= HARPsum + {{6{IIR_bass_out[17]}},IIR_bass_out} + {{6{IIR_treb_out[17]}},IIR_treb_out};
          end

        5'h09:
          begin  state <= state + 1;
          
          if ( Pluck )                                    // note that Pluck can set for only one string each DACena
            begin
            if ( Plucked_Str[4:0] == STR2 )
              begin
              Clr_Pluck <= 1;                             // clear the FF once we see it set.
              if ( Plucked_Str[5] == 1'b0 )               // bass harp pluck?
                begin                                     // A pluck event cannot occur more than once per sample time
                EX_bass_RAM[STR1]   <= 1;                 // Set excite flag to start pulse
                sign0_bass[STR0] <= ~sign0_bass[STR0];    // alternates which way the excite signal is signed.
                EXcnttmp_bass  <= PULSlen_bass ;          // length is calculated in state 7
                EXvaltmp_bass  <= ( {1'b0,VEL,10'h000} ); // Velocity used to generate pulse amplitude
                end
              else                                        // treble harp pluck
                begin
                EX_treb_RAM[STR1]   <= 1;                 // Set excite flag to start pulse
                sign0_treb[STR0] <= ~sign0_treb[STR0];    // alternates which way the excite signal is signed.
                EXcnttmp_treb  <= PULSlen_treb ;          // length is calculated in state 7
                EXvaltmp_treb  <= ( {1'b0,VEL,10'h000} ); // Velocity used to generate pulse amplitude
                end
              end
            end

          if ( SUSTAIN )
            begin
            WGin_bass <= ( sign0_bass[STR0] ) ? WGinP_bass : WGinN_bass ;
            WGin_treb <= ( sign0_treb[STR0] ) ? WGinP_treb : WGinN_treb ;
            end
          else
            begin
            WGin_bass <= ( ( ( sign0_bass[STR0] ) ? WGinP_bass : WGinN_bass ) >>> 1 ) ;
            WGin_treb <= ( ( ( sign0_treb[STR0] ) ? WGinP_treb : WGinN_treb ) >>> 1 ) ;
            end
          
          end            

        5'h0A:
          begin  state <= state + 1;
          Clr_Pluck <= 0;                       // always deassert this
          
          if ( EX_bass == 1 )                   // if this string is currently being excited:
            begin
            EXcnttmp_bass <= EXcnttmp_bass - 1; // for rectangular pulse
            if ( EXcnttmp_bass == 0 )
              begin
              EXvaltmp_bass  <= 0;               // return value to zero
              EX_bass_RAM[STR1]  <= 0;           // turn off flag excite to end pulse
              end
            end
          
          if ( EX_treb == 1 )                    // if this string is currently being excited:
            begin
            EXcnttmp_treb <= EXcnttmp_treb - 1;  // for rectangular pulse
            if ( EXcnttmp_treb == 0 )
              begin
              EXvaltmp_treb  <= 0;               // return value to zero
              EX_treb_RAM[STR1]  <= 0;           // turn off flag excite to end pulse
              end
            end

          end

        5'h0B:
          begin  state <= state + 1;
          EXval_bass[STR2] <= EXvaltmp_bass;
          EXval_treb[STR2] <= EXvaltmp_treb;

          EXcnt_bass[STR2] <= EXcnttmp_bass;
          EXcnt_treb[STR2] <= EXcnttmp_treb;

/*
          if ( oflo0 == 0 ) HARP_RAM_in <= WGin;    // No oflo, just transfer WGin to WGin              
          else
            begin                        // OFLO: apply clipping
            if ( sign0[STR0] == 1 ) HARP_RAM_in <= 18'h1FFFF ; // use positive maximum
            else                    HARP_RAM_in <= 18'h20001 ; // use negative maximum          
            end              
*/

          HARP_RAM_bass_in <= WGin_bass;
          HARP_RAM_treb_in <= WGin_treb;

          HARP_RAM_WRT <= 1;                     // set waveguide RAM write enable

          NEXT_WGadr_bass <= WGadr_bass + 1;
          NEXT_WGadr_treb <= WGadr_treb + 1;
          end


        5'h0C:
          begin
          HARP_RAM_WRT  <= 0;                     // reset waveguide RAM write enable

          WGadr_bass_RAM[STR3] <= ( NEXT_WGadr_bass <= StrEnd_bass ) ? NEXT_WGadr_bass : StrStart_bass ;
          WGadr_treb_RAM[STR3] <= ( NEXT_WGadr_treb <= StrEnd_treb ) ? NEXT_WGadr_treb : StrStart_treb ;
                     
          // if we have processed string 31, we are finished
          if ( STR == 5'd31 )  state <= state + 1;
          else                 state <= 5'h00;
          STR <= STR + 1;                        // select next string
          end

        5'h0D:
          begin
//          NEXT_DAC <= HARPsum[23:12] ;          // 64 strings
//          NEXT_DAC <= HARPsum[22:11] ;          // 32 strings
//          NEXT_DAC <= HARPsum[21:10] ;          // 16 stringgs
          NEXT_DAC <= HARPsum[20:9] ;           // 8 stringgs
//          NEXT_DAC <= HARPsum[17:6] ;           // one string at a time

          run <= 0;
          end
          
        endcase
        end
      end
    end  

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////  
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////  
// Decode structures for hardware receiving data from the MCU

  reg [7:0] SYSEX_ADDR_MSB;

  always @ ( posedge clk )
    begin
    if ( write_strobe )
      begin
// This case block contains selection logic for system level ports and for CC ports.
// Note that these ports all have the bit 7 of port_id set.
      casex ( port_id )
        8'h80:
          begin
          Plucked_Str <= out_port[5:0];          // save the string's address for the appropriate state machine
          Set_Pluck <= 1'b1;                     // set the pluck occurred FF
          end
        8'h81: VEL            <= out_port[6:0];    
        8'h82: MOD_WHL        <= out_port[6:0];     // modulation wheel, global
        8'h83: SUSTAIN        <= out_port[6];       // SUSTAIN pedal
//        8'h84: JOY_X          <= out_port[6:0];
//        8'h85: JOY_Y          <= out_port[6:0];
        8'hE0: SYSEX_ADDR_MSB <= out_port;
      endcase
      end
    else
      begin
      Set_Pluck <= 1'b0;                       // return set FF state to zero
      end
      
//////////////////////////////////////////////////////////////////////////////////////////
// These if blocks contain case blocks for sysex populated parameters     
// below this, all port_id values have bit 7 set to zero
// Address space is 14 bits.

      // SYSEX updatable parameter ports:    

    if ( write_strobe )        
      begin
      case( SYSEX_ADDR_MSB )
        
        8'h00:
          begin
          casex ( port_id )
            8'h00: MIDI_CHAN                                 <= out_port[3:0];
          endcase
          end          
          
        8'h01:              // PAGE 01
          begin
          casex ( port_id )
            8'b000xxxxx: a0_bass_lo_RAM[port_id[4:0]]             <= out_port[6:0]; // 00 - 03
            8'b001xxxxx: a0_treb_lo_RAM[port_id[4:0]]             <= out_port[6:0]; // 00 - 03
          endcase
          end

        8'h02:              // PAGE 02
          begin
          casex ( port_id )
            8'b000xxxxx: a0_bass_mi_RAM[port_id[4:0]]             <= out_port[6:0]; // 00 - 03
            8'b001xxxxx: a0_treb_mi_RAM[port_id[4:0]]             <= out_port[6:0]; // 00 - 03
          endcase
          end

        8'h03:              // PAGE 03
          begin
          casex ( port_id )
            8'b000xxxxx: a0_bass_hi_RAM[port_id[4:0]]             <= out_port[3:0]; // 00 - 03
            8'b001xxxxx: a0_treb_hi_RAM[port_id[4:0]]             <= out_port[3:0]; // 00 - 03
          endcase
          end

      endcase
      end
    end

// Instantiate the single Pluck_FF RS flip-flop
  FDCPE #(.INIT(1'b0)) Pluck_FF ( .Q( Pluck ), .C( 1'b0 ), .CE( 1'b0 ), .CLR( Clr_Pluck ), .D( 1'b0 ), .PRE( Set_Pluck ) );

// INPUTS TO MIDI CONTROLLER
  always @ ( posedge clk ) 
    begin
    case ( port_id[3:0] )                           // decode and transfer data to in_port_reg
      4'h0:    in_port_reg <= {rx1ready_status,rx0ready_status,6'b000000}; // UART1 & UART0 rxready bits
      4'h1:    in_port_reg <= rx0data;              // MIDI UART rxdata
      4'h2:    in_port_reg <= {4'b0000,MIDI_CHAN};  // slide switches, lower 4 used to select MIDI chan.
      4'h3:    in_port_reg <= 8'd36;         // TRANSPOSE !!!  The MIDI controller will subtract this value 
                                             // from every note number received.  Use this to ensure that the 
                                             // lowest keyboard key plays the lowest string.
      4'h9:    in_port_reg <= rx1data;                     // TTY UART rxdata
      
      default: in_port_reg <= 8'bxxxxxxxx;
    endcase
    end

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
// LEDs


//assign led = ( no_leds ) ? 8'h00 : {SUSTAIN,~Raw_MIDI_In,Plucked_Str};   // default
assign led = ( no_leds ) ? 8'h00 : {~TTY_In,~Raw_MIDI_In,Plucked_Str};

// DIAGNOSTIC:
//  assign led = port_id;    // hook LEDs to something to eliminate errors

//assign led = EX;
//assign led = {SUSTAIN,~MIDI_In,Plucked_Str,Plucked_MH};

//assign led = {PB,4'b0000};

//assign led = DIAG;
//always @ ( posedge clk )
//  begin
//  if ( Set_Pluck ) DIAG <= DIAG + 1;
//  end

endmodule
